package com.gic.spark.tag;

import com.gic.spark.entity.TagConditionDTO;
import com.gic.spark.entity.TagConditionValDTO;
import com.gic.spark.entity.enumtype.CouponType;
import com.gic.spark.entity.enumtype.DomainType;
import com.gic.spark.entity.enumtype.NumberType;
import com.gic.spark.entity.enumtype.TimeRangeType;
import com.gic.spark.entity.request.CouponRequest;
import com.gic.spark.entity.request.FilterRequest;
import com.gic.spark.entity.request.IntegralRequest;
import com.gic.spark.util.DateUtil;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @description:
 * @author: wangxk
 * @date: 2020/4/15
 */
public class TagValueParser {

    public static FilterRequest parseFilterValue(TagConditionDTO conditionDTO){
        FilterRequest request=null;
        switch (conditionDTO.getTagEsFieldName()){
            //积分
            case TagConstant.TAG_CODE_ACCUMULATED_INTEGRAL:
                request=getIntegralRequest(conditionDTO.getTagTemplateElInfo());
                break;
            case TagConstant.TAG_CODE_ABOUT_EXPIRE_INTEGRAL:
                request=getIntegralRequest(conditionDTO.getTagTemplateElInfo());
                break;

            //卡卷
            case TagConstant.TAG_CODE_CURRENT_COUPON_NUM:
                request=getCurrentCouponNumRequest(conditionDTO.getTagTemplateElInfo());
                break;
            case TagConstant.TAG_CODE_UNCLAIMED_COUPON:
                request=getCouponRequest(conditionDTO);
                break;
            case TagConstant.TAG_CODE_GET_NO_CANCEL_COUPON:
                request=getCouponRequest(conditionDTO);
                break;
            case TagConstant.TAG_CODE_NO_CANCEL_EXPIRES_COUPON:
                request=getCouponRequest(conditionDTO);
                break;
            case TagConstant.TAG_CODE_CANCEL_COUPON:
                request=getCouponRequest(conditionDTO);
                break;
            case TagConstant.TAG_CODE_OCCUPY_COUPON:
                request=getCouponRequest(conditionDTO);
                break;
            case TagConstant.TAG_CODE_DONATION_IN_COUPON:
                request=getCouponRequest(conditionDTO);
                break;
            case TagConstant.TAG_CODE_YET_DONATION_COUPON:
                request=getCouponRequest(conditionDTO);
                break;

            default:break;
        }
        return request;
    }

    private static FilterRequest getCouponRequest(TagConditionDTO conditionDTO) {
        CouponRequest request=new CouponRequest();
        for(TagConditionValDTO conditionValDTO:conditionDTO.getTagTemplateElInfo()){
            switch (conditionValDTO.getKey()){
                case TagConstant.TAG_COMPONENT_COUPON:
                    switch (conditionDTO.getTagEsFieldName()){
                        case TagConstant.TAG_CODE_UNCLAIMED_COUPON:
                            request.setCouponType(CouponType.UNCLAIMED);
                            break;
                        case TagConstant.TAG_CODE_GET_NO_CANCEL_COUPON:
                            request.setCouponType(CouponType.NO_CANCEL);
                            break;
                        case TagConstant.TAG_CODE_NO_CANCEL_EXPIRES_COUPON:
                            request.setCouponType(CouponType.NO_CANCEL_EXPIRES);
                            break;
                        case TagConstant.TAG_CODE_CANCEL_COUPON:
                            request.setCouponType(CouponType.CANCEL);
                            break;
                        case TagConstant.TAG_CODE_OCCUPY_COUPON:
                            request.setCouponType(CouponType.OCCUPY);
                            break;
                        case TagConstant.TAG_CODE_DONATION_IN_COUPON:
                            request.setCouponType(CouponType.DONATION_IN);
                            break;
                        case TagConstant.TAG_CODE_YET_DONATION_COUPON:
                            request.setCouponType(CouponType.YET_DONATION);
                            break;
                        default:break;
                    }
                    List<Integer>couponList=new ArrayList();
                    if(StringUtils.isNotEmpty(conditionValDTO.getVal())){
                        String[] couponIds= conditionValDTO.getVal().split(",");
                        for (String couponId : couponIds) {
                            couponList.add(Integer.valueOf(couponId));
                        }
                    }
                    request.setCouponList(couponList);
                    break;
                default:break;
            }
        }
        return request;
    }

    private static FilterRequest getCurrentCouponNumRequest(List<TagConditionValDTO> conditionValDTOList) {
        CouponRequest request=new CouponRequest();
        for(TagConditionValDTO conditionValDTO:conditionValDTOList){
            String[] keys=conditionValDTO.getKey().split(".");
            if(keys.length==3){
                setDomainHandle(request,keys[1],conditionValDTO.getVal());
            }else{
                switch (keys[0]){
                    case TagConstant.TAG_KEY_TYPE_NUMBER:
                        switch (keys[1]){
                            case TagConstant.TAG_COMPUTE_TYPE_GT:
                                request.setBeginNum(Integer.parseInt(conditionValDTO.getVal()));
                                request.setNumberType(NumberType.gt);
                                break;
                            case TagConstant.TAG_COMPUTE_TYPE_GTE:
                                request.setBeginNum(Integer.parseInt(conditionValDTO.getVal()));
                                request.setNumberType(NumberType.gte);
                                break;
                            case TagConstant.TAG_COMPUTE_TYPE_LT:
                                request.setEndNum(Integer.parseInt(conditionValDTO.getVal()));
                                request.setNumberType(NumberType.lt);
                                break;
                            case TagConstant.TAG_COMPUTE_TYPE_LTE:
                                request.setEndNum(Integer.parseInt(conditionValDTO.getVal()));
                                request.setNumberType(NumberType.lte);
                                break;
                            case TagConstant.TAG_COMPUTE_TYPE_EQ:
                                request.setEqualNum(Integer.parseInt(conditionValDTO.getVal()));
                                request.setNumberType(NumberType.eq);
                                break;
                            case TagConstant.TAG_COMPUTE_TYPE_BETWEEN:
                                String[] vals=conditionValDTO.getVal().split(",");
                                request.setBeginNum(Integer.parseInt(vals[0]));
                                request.setEndNum(Integer.parseInt(vals[1]));
                                request.setNumberType(NumberType.between);
                                break;
                            default:break;
                        }
                        break;
                    default:break;
                }
            }
        }
        return request;
    }

    private static FilterRequest getIntegralRequest(List<TagConditionValDTO> conditionValDTOList) {
        IntegralRequest request=new IntegralRequest();
        for(TagConditionValDTO conditionValDTO:conditionValDTOList){
            String[] keys=conditionValDTO.getKey().split(".");
            if(keys.length==1){
                setDomainHandle(request,keys[0],conditionValDTO.getVal());
            }else{
                switch (keys[0]){
                    case TagConstant.TAG_KEY_TYPE_TIME_RANGE:
                        switch (keys[1]){
                            case TagConstant.TAG_TIME_TYPE_FIXATION:
                                String[] vals=conditionValDTO.getVal().split(",");
                                request.setBeginTime(new Timestamp(DateUtil.stringToDate(vals[0]).getTime()));
                                request.setEndTime(new Timestamp(DateUtil.stringToDate(vals[1]).getTime()));
                                request.setTimeRangeType(TimeRangeType.FIXATION);
                                break;
                            case TagConstant.TAG_TIME_TYPE_LATELY:
                                request.setTimeNum(Integer.parseInt(conditionValDTO.getVal()));
                                request.setTimeRangeType(TimeRangeType.LATELY);
                                break;
                            case TagConstant.TAG_TIME_TYPE_LATER:
                                request.setTimeNum(Integer.parseInt(conditionValDTO.getVal()));
                                request.setTimeRangeType(TimeRangeType.LATER);
                                break;
                            default:break;
                        }
                        break;
                    case  TagConstant.TAG_KEY_TYPE_NUMBER:
                        switch (keys[1]){
                            case TagConstant.TAG_COMPUTE_TYPE_GT:
                                request.setBeginNum(Integer.parseInt(conditionValDTO.getVal()));
                                request.setNumberType(NumberType.gt);
                                break;
                            case TagConstant.TAG_COMPUTE_TYPE_GTE:
                                request.setBeginNum(Integer.parseInt(conditionValDTO.getVal()));
                                request.setNumberType(NumberType.gte);
                                break;
                            case TagConstant.TAG_COMPUTE_TYPE_LT:
                                request.setEndNum(Integer.parseInt(conditionValDTO.getVal()));
                                request.setNumberType(NumberType.lt);
                                break;
                            case TagConstant.TAG_COMPUTE_TYPE_LTE:
                                request.setEndNum(Integer.parseInt(conditionValDTO.getVal()));
                                request.setNumberType(NumberType.lte);
                                break;
                            case TagConstant.TAG_COMPUTE_TYPE_EQ:
                                request.setEqualNum(Integer.parseInt(conditionValDTO.getVal()));
                                request.setNumberType(NumberType.eq);
                                break;
                            case TagConstant.TAG_COMPUTE_TYPE_BETWEEN:
                                String[] vals=conditionValDTO.getVal().split(",");
                                request.setBeginNum(Integer.parseInt(vals[0]));
                                request.setEndNum(Integer.parseInt(vals[1]));
                                request.setNumberType(NumberType.between);
                                break;
                            default:break;
                        }
                    default:break;
                }
            }
        }
        return request;
    }

    private static void setDomainHandle(FilterRequest request,String key,String val){
        switch (key){
            case TagConstant.TAG_DOMAIN_ECU_INFO:
                request.setDomainType(DomainType.ECU_INFO);
                break;
            case TagConstant.TAG_DOMAIN_ACU_INFO:
                request.setDomainType(DomainType.ACU_INFO);
                break;
            case TagConstant.TAG_DOMAIN_SCU_INFO:
                request.setDomainType(DomainType.SCU_INFO);
                break;
            case TagConstant.TAG_DOMAIN_MCU_INFO:
                request.setDomainType(DomainType.MCU_INFO);
                break;
            default:break;
        }
        String[] vals=val.split(",");
        request.setCuVals(Arrays.asList(vals));
    }
}
